@@ -17,8 +17,8 @@
 
 import argparse
 import io
+import logging
 import os
-import six
 import sys
 
 if os.name == "nt":
@@ -27,10 +27,12 @@ else:
     msvcrt = None
 
 from glanceclient.common import utils as gc_utils
+from osc_lib.cli import parseractions
+from osc_lib.command import command
+from osc_lib import utils
+import six
+
 from openstackclient.api import utils as api_utils
-from openstackclient.common import command
-from openstackclient.common import parseractions
-from openstackclient.common import utils
 from openstackclient.i18n import _
 
 
@@ -38,6 +40,9 @@ DEFAULT_CONTAINER_FORMAT = 'bare'
 DEFAULT_DISK_FORMAT = 'raw'
 
 
+LOG = logging.getLogger(__name__)
+
+
 def _format_visibility(data):
     """Return a formatted visibility string
 
@@ -188,10 +193,8 @@ class CreateImage(command.ShowOne):
         image_client = self.app.client_manager.image
 
         if getattr(parsed_args, 'owner', None) is not None:
-            self.log.warning(_(
-                'The --owner option is deprecated, '
-                'please use --project instead.'
-            ))
+            LOG.warning(_('The --owner option is deprecated, '
+                          'please use --project instead.'))
 
         # Build an attribute dict from the parsed args, only include
         # attributes that were actually set on the command line
@@ -375,6 +378,7 @@ class ListImage(command.Lister):
                 'Disk Format',
                 'Container Format',
                 'Size',
+                'Checksum',
                 'Status',
                 'is_public',
                 'protected',
@@ -387,6 +391,7 @@ class ListImage(command.Lister):
                 'Disk Format',
                 'Container Format',
                 'Size',
+                'Checksum',
                 'Status',
                 'Visibility',
                 'Protected',
@@ -607,10 +612,8 @@ class SetImage(command.Command):
         image_client = self.app.client_manager.image
 
         if getattr(parsed_args, 'owner', None) is not None:
-            self.log.warning(_(
-                'The --owner option is deprecated, '
-                'please use --project instead.'
-            ))
+            LOG.warning(_('The --owner option is deprecated, '
+                          'please use --project instead.'))
 
         kwargs = {}
         copy_attrs = ('name', 'owner', 'min_disk', 'min_ram', 'properties',
@@ -683,18 +686,13 @@ class SetImage(command.Command):
                             # will do a chunked transfer
                             kwargs["data"] = sys.stdin
                         else:
-                            self.log.warning(_('Use --stdin to enable read '
-                                               'image data from standard '
-                                               'input'))
+                            LOG.warning(_('Use --stdin to enable read image '
+                                          'data from standard input'))
 
             if image.properties and parsed_args.properties:
                 image.properties.update(kwargs['properties'])
                 kwargs['properties'] = image.properties
 
-            if not kwargs:
-                self.log.warning('no arguments specified')
-                return
-
             image = image_client.images.update(image.id, **kwargs)
         finally:
             # Clean up open files - make sure data isn't a string
