@@ -28,9 +28,14 @@ from openstackclient.identity import common as identity_common
 LOG = logging.getLogger(__name__)
 
 
-def _update_arguments(obj_list, parsed_args_list):
+def _update_arguments(obj_list, parsed_args_list, option):
     for item in parsed_args_list:
-        obj_list.remove(item)
+        try:
+            obj_list.remove(item)
+        except ValueError:
+            msg = (_("Subnet does not contain %(option)s %(value)s") %
+                   {'option': option, 'value': item})
+            raise exceptions.CommandError(msg)
 
 
 def _format_allocation_pools(data):
@@ -48,10 +53,11 @@ _formatters = {
     'allocation_pools': _format_allocation_pools,
     'dns_nameservers': utils.format_list,
     'host_routes': _format_host_routes,
+    'service_types': utils.format_list,
 }
 
 
-def _get_common_parse_arguments(parser):
+def _get_common_parse_arguments(parser, is_create=True):
     parser.add_argument(
         '--allocation-pool',
         metavar='start=<ip-address>,end=<ip-address>',
@@ -62,6 +68,14 @@ def _get_common_parse_arguments(parser):
                "e.g.: start=192.168.199.2,end=192.168.199.254 "
                "(repeat option to add multiple IP addresses)")
     )
+    if not is_create:
+        parser.add_argument(
+            '--no-allocation-pool',
+            action='store_true',
+            help=_("Clear associated allocation-pools from the subnet. "
+                   "Specify both --allocation-pool and --no-allocation-pool "
+                   "to overwrite the current allocation pool information.")
+        )
     parser.add_argument(
         '--dns-nameserver',
         metavar='<dns-nameserver>',
@@ -82,6 +96,24 @@ def _get_common_parse_arguments(parser):
                "gateway: nexthop IP address "
                "(repeat option to add multiple routes)")
     )
+    if not is_create:
+        parser.add_argument(
+            '--no-host-route',
+            action='store_true',
+            help=_("Clear associated host-routes from the subnet. "
+                   "Specify both --host-route and --no-host-route "
+                   "to overwrite the current host route information.")
+        )
+    parser.add_argument(
+        '--service-type',
+        metavar='<service-type>',
+        action='append',
+        dest='service_types',
+        help=_("Service type for this subnet "
+               "e.g.: network:floatingip_agent_gateway. "
+               "Must be a valid device owner value for a network port "
+               "(repeat option to set multiple service types)")
+    )
 
 
 def _get_columns(item):
@@ -147,7 +179,7 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
             attrs['ipv6_ra_mode'] = parsed_args.ipv6_ra_mode
         if parsed_args.ipv6_address_mode is not None:
             attrs['ipv6_address_mode'] = parsed_args.ipv6_address_mode
-        if 'network_segment' in parsed_args:
+        if parsed_args.network_segment is not None:
             attrs['segment_id'] = client.find_segment(
                 parsed_args.network_segment, ignore_missing=False).id
 
@@ -177,6 +209,11 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
         # Change 'gateway' entry to 'nexthop' to match the API
         attrs['host_routes'] = convert_entries_to_nexthop(
             parsed_args.host_routes)
+    if ('service_types' in parsed_args and
+       parsed_args.service_types is not None):
+        attrs['service_types'] = parsed_args.service_types
+    if parsed_args.description is not None:
+        attrs['description'] = parsed_args.description
     return attrs
 
 
@@ -262,19 +299,23 @@ class CreateSubnet(command.ShowOne):
             help=_("IPv6 address mode, "
                    "valid modes: [dhcpv6-stateful, dhcpv6-stateless, slaac]")
         )
-        if self.app.options.os_beta_command:
-            parser.add_argument(
-                '--network-segment',
-                metavar='<network-segment>',
-                help=_("Network segment to associate with this subnet "
-                       "(ID only)")
-            )
+        parser.add_argument(
+            '--network-segment',
+            metavar='<network-segment>',
+            help=_("Network segment to associate with this subnet "
+                   "(name or ID)")
+        )
         parser.add_argument(
             '--network',
             required=True,
             metavar='<network>',
             help=_("Network this subnet belongs to (name or ID)")
         )
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            help=_("Set subnet description")
+        )
         _get_common_parse_arguments(parser)
         return parser
 
@@ -352,9 +393,51 @@ class ListSubnet(command.Lister):
             action='store_true',
             help=_("List subnets which have DHCP disabled")
         )
+        parser.add_argument(
+            '--service-type',
+            metavar='<service-type>',
+            action='append',
+            dest='service_types',
+            help=_("List only subnets of a given service type in output "
+                   "e.g.: network:floatingip_agent_gateway. "
+                   "Must be a valid device owner value for a network port "
+                   "(repeat option to list multiple service types)")
+        )
+        parser.add_argument(
+            '--project',
+            metavar='<project>',
+            help=_("List only subnets which belong to a given project "
+                   "(name or ID) in output")
+        )
+        identity_common.add_project_domain_option_to_parser(parser)
+        parser.add_argument(
+            '--network',
+            metavar='<network>',
+            help=_("List only subnets which belong to a given network "
+                   "(name or ID) in output")
+        )
+        parser.add_argument(
+            '--gateway',
+            metavar='<gateway>',
+            help=_("List only subnets of given gateway IP in output")
+        )
+        parser.add_argument(
+            '--name',
+            metavar='<name>',
+            help=_("List only subnets of given name in output")
+        )
+        parser.add_argument(
+            '--subnet-range',
+            metavar='<subnet-range>',
+            help=_("List only subnets of given subnet range "
+                   "(in CIDR notation) in output "
+                   "e.g.: --subnet-range 10.10.0.0/16")
+        )
         return parser
 
     def take_action(self, parsed_args):
+        identity_client = self.app.client_manager.identity
+        network_client = self.app.client_manager.network
         filters = {}
         if parsed_args.ip_version:
             filters['ip_version'] = parsed_args.ip_version
@@ -362,17 +445,36 @@ class ListSubnet(command.Lister):
             filters['enable_dhcp'] = True
         elif parsed_args.no_dhcp:
             filters['enable_dhcp'] = False
-        data = self.app.client_manager.network.subnets(**filters)
+        if parsed_args.service_types:
+            filters['service_types'] = parsed_args.service_types
+        if parsed_args.project:
+            project_id = identity_common.find_project(
+                identity_client,
+                parsed_args.project,
+                parsed_args.project_domain,
+            ).id
+            filters['tenant_id'] = project_id
+        if parsed_args.network:
+            network_id = network_client.find_network(parsed_args.network,
+                                                     ignore_missing=False).id
+            filters['network_id'] = network_id
+        if parsed_args.gateway:
+            filters['gateway_ip'] = parsed_args.gateway
+        if parsed_args.name:
+            filters['name'] = parsed_args.name
+        if parsed_args.subnet_range:
+            filters['cidr'] = parsed_args.subnet_range
+        data = network_client.subnets(**filters)
 
         headers = ('ID', 'Name', 'Network', 'Subnet')
         columns = ('id', 'name', 'network_id', 'cidr')
         if parsed_args.long:
             headers += ('Project', 'DHCP', 'Name Servers',
                         'Allocation Pools', 'Host Routes', 'IP Version',
-                        'Gateway')
+                        'Gateway', 'Service Types')
             columns += ('tenant_id', 'enable_dhcp', 'dns_nameservers',
                         'allocation_pools', 'host_routes', 'ip_version',
-                        'gateway_ip')
+                        'gateway_ip', 'service_types')
 
         return (headers,
                 (utils.get_item_properties(
@@ -416,7 +518,12 @@ class SetSubnet(command.Command):
                    "'none': This subnet will not use a gateway, "
                    "e.g.: --gateway 192.168.9.1, --gateway none.")
         )
-        _get_common_parse_arguments(parser)
+        parser.add_argument(
+            '--description',
+            metavar='<description>',
+            help=_("Set subnet description")
+        )
+        _get_common_parse_arguments(parser, is_create=False)
         return parser
 
     def take_action(self, parsed_args):
@@ -427,9 +534,17 @@ class SetSubnet(command.Command):
         if 'dns_nameservers' in attrs:
             attrs['dns_nameservers'] += obj.dns_nameservers
         if 'host_routes' in attrs:
-            attrs['host_routes'] += obj.host_routes
+            if not parsed_args.no_host_route:
+                attrs['host_routes'] += obj.host_routes
+        elif parsed_args.no_host_route:
+            attrs['host_routes'] = ''
         if 'allocation_pools' in attrs:
-            attrs['allocation_pools'] += obj.allocation_pools
+            if not parsed_args.no_allocation_pool:
+                attrs['allocation_pools'] += obj.allocation_pools
+        elif parsed_args.no_allocation_pool:
+            attrs['allocation_pools'] = ''
+        if 'service_types' in attrs:
+            attrs['service_types'] += obj.service_types
         client.update_subnet(obj, **attrs)
         return
 
@@ -465,9 +580,9 @@ class UnsetSubnet(command.Command):
             dest='allocation_pools',
             action=parseractions.MultiKeyValueAction,
             required_keys=['start', 'end'],
-            help=_('Allocation pool to be removed from this subnet '
-                   'e.g.: start=192.168.199.2,end=192.168.199.254 '
-                   '(repeat option to unset multiple Allocation pools)')
+            help=_('Allocation pool IP addresses to be removed from this '
+                   'subnet e.g.: start=192.168.199.2,end=192.168.199.254 '
+                   '(repeat option to unset multiple allocation pools)')
         )
         parser.add_argument(
             '--dns-nameserver',
@@ -475,7 +590,7 @@ class UnsetSubnet(command.Command):
             action='append',
             dest='dns_nameservers',
             help=_('DNS server to be removed from this subnet '
-                   '(repeat option to set multiple DNS servers)')
+                   '(repeat option to unset multiple DNS servers)')
         )
         parser.add_argument(
             '--host-route',
@@ -489,6 +604,16 @@ class UnsetSubnet(command.Command):
                    'gateway: nexthop IP address '
                    '(repeat option to unset multiple host routes)')
         )
+        parser.add_argument(
+            '--service-type',
+            metavar='<service-type>',
+            action='append',
+            dest='service_types',
+            help=_('Service type to be removed from this subnet '
+                   'e.g.: network:floatingip_agent_gateway. '
+                   'Must be a valid device owner value for a network port '
+                   '(repeat option to unset multiple service types)')
+        )
         parser.add_argument(
             'subnet',
             metavar="<subnet>",
@@ -502,31 +627,25 @@ class UnsetSubnet(command.Command):
         tmp_obj = copy.deepcopy(obj)
         attrs = {}
         if parsed_args.dns_nameservers:
-            try:
-                _update_arguments(tmp_obj.dns_nameservers,
-                                  parsed_args.dns_nameservers)
-            except ValueError as error:
-                msg = (_("%s not in dns-nameservers") % str(error))
-                raise exceptions.CommandError(msg)
+            _update_arguments(tmp_obj.dns_nameservers,
+                              parsed_args.dns_nameservers,
+                              'dns-nameserver')
             attrs['dns_nameservers'] = tmp_obj.dns_nameservers
         if parsed_args.host_routes:
-            try:
-                _update_arguments(
-                    tmp_obj.host_routes,
-                    convert_entries_to_nexthop(parsed_args.host_routes))
-            except ValueError as error:
-                msg = (_("Subnet does not have %s in host-routes") %
-                       str(error))
-                raise exceptions.CommandError(msg)
+            _update_arguments(
+                tmp_obj.host_routes,
+                convert_entries_to_nexthop(parsed_args.host_routes),
+                'host-route')
             attrs['host_routes'] = tmp_obj.host_routes
         if parsed_args.allocation_pools:
-            try:
-                _update_arguments(tmp_obj.allocation_pools,
-                                  parsed_args.allocation_pools)
-            except ValueError as error:
-                msg = (_("Subnet does not have %s in allocation-pools") %
-                       str(error))
-                raise exceptions.CommandError(msg)
+            _update_arguments(tmp_obj.allocation_pools,
+                              parsed_args.allocation_pools,
+                              'allocation-pool')
             attrs['allocation_pools'] = tmp_obj.allocation_pools
+        if parsed_args.service_types:
+            _update_arguments(tmp_obj.service_types,
+                              parsed_args.service_types,
+                              'service-type')
+            attrs['service_types'] = tmp_obj.service_types
         if attrs:
             client.update_subnet(obj, **attrs)
