@@ -1,4 +1,4 @@
-#   Copyright 2012-2013 OpenStack, LLC.
+#   Copyright 2012-2013 OpenStack Foundation
 #
 #   Licensed under the Apache License, Version 2.0 (the "License"); you may
 #   not use this file except in compliance with the License. You may obtain
@@ -16,14 +16,20 @@
 """Volume v1 Snapshot action implementations"""
 
 import copy
+import logging
+
+from osc_lib.cli import parseractions
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
 import six
 
-from openstackclient.common import command
-from openstackclient.common import parseractions
-from openstackclient.common import utils
 from openstackclient.i18n import _
 
 
+LOG = logging.getLogger(__name__)
+
+
 class CreateSnapshot(command.ShowOne):
     """Create new snapshot"""
 
@@ -87,10 +93,24 @@ class DeleteSnapshot(command.Command):
 
     def take_action(self, parsed_args):
         volume_client = self.app.client_manager.volume
-        for snapshot in parsed_args.snapshots:
-            snapshot_id = utils.find_resource(volume_client.volume_snapshots,
-                                              snapshot).id
-            volume_client.volume_snapshots.delete(snapshot_id)
+        result = 0
+
+        for i in parsed_args.snapshots:
+            try:
+                snapshot_id = utils.find_resource(
+                    volume_client.volume_snapshots, i).id
+                volume_client.volume_snapshots.delete(snapshot_id)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete snapshot with "
+                            "name or ID '%(snapshot)s': %(e)s"),
+                          {'snapshot': i, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.snapshots)
+            msg = (_("%(result)s of %(total)s snapshots failed "
+                   "to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListSnapshot(command.Lister):
@@ -198,17 +218,31 @@ class SetSnapshot(command.Command):
         snapshot = utils.find_resource(volume_client.volume_snapshots,
                                        parsed_args.snapshot)
 
+        result = 0
         if parsed_args.property:
-            volume_client.volume_snapshots.set_metadata(snapshot.id,
-                                                        parsed_args.property)
+            try:
+                volume_client.volume_snapshots.set_metadata(
+                    snapshot.id, parsed_args.property)
+            except Exception as e:
+                LOG.error(_("Failed to set snapshot property: %s"), e)
+                result += 1
 
         kwargs = {}
         if parsed_args.name:
             kwargs['display_name'] = parsed_args.name
         if parsed_args.description:
             kwargs['display_description'] = parsed_args.description
+        if kwargs:
+            try:
+                snapshot.update(**kwargs)
+            except Exception as e:
+                LOG.error(_("Failed to update snapshot display name "
+                          "or display description: %s"), e)
+                result += 1
 
-        snapshot.update(**kwargs)
+        if result > 0:
+            raise exceptions.CommandError(_("One or more of the "
+                                          "set operations failed"))
 
 
 class ShowSnapshot(command.ShowOne):
@@ -249,8 +283,6 @@ class UnsetSnapshot(command.Command):
             '--property',
             metavar='<key>',
             action='append',
-            default=[],
-            required=True,
             help=_('Property to remove from snapshot '
                    '(repeat option to remove multiple properties)'),
         )
