@@ -30,6 +30,20 @@ from openstackclient.identity import common as identity_common
 LOG = logging.getLogger(__name__)
 
 
+def _check_size_arg(args):
+    """Check whether --size option is required or not.
+
+    Require size parameter only in case when snapshot or source
+    volume is not specified.
+    """
+
+    if ((args.snapshot or args.source or args.source_replicated)
+            is None and args.size is None):
+        msg = _("--size is a required option if snapshot "
+                "or source volume is not specified.")
+        raise exceptions.CommandError(msg)
+
+
 class CreateVolume(command.ShowOne):
     """Create new volume"""
 
@@ -44,29 +58,35 @@ class CreateVolume(command.ShowOne):
             "--size",
             metavar="<size>",
             type=int,
-            required=True,
-            help=_("Volume size in GB"),
+            help=_("Volume size in GB (Required unless --snapshot or "
+                   "--source or --source-replicated is specified)"),
         )
         parser.add_argument(
             "--type",
             metavar="<volume-type>",
             help=_("Set the type of volume"),
         )
-        parser.add_argument(
+        source_group = parser.add_mutually_exclusive_group()
+        source_group.add_argument(
             "--image",
             metavar="<image>",
             help=_("Use <image> as source of volume (name or ID)"),
         )
-        parser.add_argument(
+        source_group.add_argument(
             "--snapshot",
             metavar="<snapshot>",
             help=_("Use <snapshot> as source of volume (name or ID)"),
         )
-        parser.add_argument(
+        source_group.add_argument(
             "--source",
             metavar="<volume>",
             help=_("Volume to clone (name or ID)"),
         )
+        source_group.add_argument(
+            "--source-replicated",
+            metavar="<replicated-volume>",
+            help=_("Replicated volume to clone (name or ID)"),
+        )
         parser.add_argument(
             "--description",
             metavar="<description>",
@@ -87,6 +107,11 @@ class CreateVolume(command.ShowOne):
             metavar="<availability-zone>",
             help=_("Create volume in <availability-zone>"),
         )
+        parser.add_argument(
+            "--consistency-group",
+            metavar="consistency-group>",
+            help=_("Consistency group where the new volume belongs to"),
+        )
         parser.add_argument(
             "--property",
             metavar="<key=value>",
@@ -94,9 +119,23 @@ class CreateVolume(command.ShowOne):
             help=_("Set a property to this volume "
                    "(repeat option to set multiple properties)"),
         )
+        parser.add_argument(
+            "--hint",
+            metavar="<key=value>",
+            action=parseractions.KeyValueAction,
+            help=_("Arbitrary scheduler hint key-value pairs to help boot "
+                   "an instance (repeat option to set multiple hints)"),
+        )
+        parser.add_argument(
+            "--multi-attach",
+            action="store_true",
+            help=_("Allow volume to be attached more than once "
+                   "(default to False)")
+        )
         return parser
 
     def take_action(self, parsed_args):
+        _check_size_arg(parsed_args)
         identity_client = self.app.client_manager.identity
         volume_client = self.app.client_manager.volume
         image_client = self.app.client_manager.image
@@ -107,6 +146,18 @@ class CreateVolume(command.ShowOne):
                 volume_client.volumes,
                 parsed_args.source).id
 
+        replicated_source_volume = None
+        if parsed_args.source_replicated:
+            replicated_source_volume = utils.find_resource(
+                volume_client.volumes,
+                parsed_args.source_replicated).id
+
+        consistency_group = None
+        if parsed_args.consistency_group:
+            consistency_group = utils.find_resource(
+                volume_client.consistencygroups,
+                parsed_args.consistency_group).id
+
         image = None
         if parsed_args.image:
             image = utils.find_resource(
@@ -142,7 +193,11 @@ class CreateVolume(command.ShowOne):
             availability_zone=parsed_args.availability_zone,
             metadata=parsed_args.property,
             imageRef=image,
-            source_volid=source_volume
+            source_volid=source_volume,
+            consistencygroup_id=consistency_group,
+            source_replica=replicated_source_volume,
+            multiattach=parsed_args.multi_attach,
+            scheduler_hints=parsed_args.hint,
         )
         # Remove key links from being displayed
         volume._info.update(
@@ -246,6 +301,18 @@ class ListVolume(command.Lister):
             default=False,
             help=_('List additional fields in output'),
         )
+        parser.add_argument(
+            '--marker',
+            metavar='<marker>',
+            help=_('The last volume ID of the previous page'),
+        )
+        parser.add_argument(
+            '--limit',
+            type=int,
+            action=parseractions.NonNegativeAction,
+            metavar='<limit>',
+            help=_('Maximum number of volumes to display'),
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -328,7 +395,11 @@ class ListVolume(command.Lister):
             'status': parsed_args.status,
         }
 
-        data = volume_client.volumes.list(search_opts=search_opts)
+        data = volume_client.volumes.list(
+            search_opts=search_opts,
+            marker=parsed_args.marker,
+            limit=parsed_args.limit,
+        )
 
         return (column_headers,
                 (utils.get_item_properties(
@@ -386,7 +457,21 @@ class SetVolume(command.Command):
                      'maintenance'],
             help=_('New volume state ("available", "error", "creating", '
                    '"deleting", "in-use", "attaching", "detaching", '
-                   '"error_deleting" or "maintenance")'),
+                   '"error_deleting" or "maintenance") (admin only) '
+                   '(This option simply changes the state of the volume '
+                   'in the database with no regard to actual status, '
+                   'exercise caution when using)'),
+        )
+        bootable_group = parser.add_mutually_exclusive_group()
+        bootable_group.add_argument(
+            "--bootable",
+            action="store_true",
+            help=_("Mark volume as bootable")
+        )
+        bootable_group.add_argument(
+            "--non-bootable",
+            action="store_true",
+            help=_("Mark volume as non-bootable")
         )
         return parser
 
@@ -430,6 +515,13 @@ class SetVolume(command.Command):
             except Exception as e:
                 LOG.error(_("Failed to set volume state: %s"), e)
                 result += 1
+        if parsed_args.bootable or parsed_args.non_bootable:
+            try:
+                volume_client.volumes.set_bootable(
+                    volume.id, parsed_args.bootable)
+            except Exception as e:
+                LOG.error(_("Failed to set volume bootable property: %s"), e)
+                result += 1
 
         kwargs = {}
         if parsed_args.name:
@@ -456,7 +548,7 @@ class ShowVolume(command.ShowOne):
         parser = super(ShowVolume, self).get_parser(prog_name)
         parser.add_argument(
             'volume',
-            metavar="<volume-id>",
+            metavar="<volume>",
             help=_("Volume to display (name or ID)")
         )
         return parser
@@ -511,9 +603,23 @@ class UnsetVolume(command.Command):
         volume = utils.find_resource(
             volume_client.volumes, parsed_args.volume)
 
+        result = 0
         if parsed_args.property:
-            volume_client.volumes.delete_metadata(
-                volume.id, parsed_args.property)
+            try:
+                volume_client.volumes.delete_metadata(
+                    volume.id, parsed_args.property)
+            except Exception as e:
+                LOG.error(_("Failed to unset volume property: %s"), e)
+                result += 1
+
         if parsed_args.image_property:
-            volume_client.volumes.delete_image_metadata(
-                volume.id, parsed_args.image_property)
+            try:
+                volume_client.volumes.delete_image_metadata(
+                    volume.id, parsed_args.image_property)
+            except Exception as e:
+                LOG.error(_("Failed to unset image property: %s"), e)
+                result += 1
+
+        if result > 0:
+            raise exceptions.CommandError(_("One or more of the "
+                                          "unset operations failed"))
